/**
 * Fast Matrix Exponentiation
 *
 * Complexity (Time): O(K^3 log n)
 * Complexity (Space): O(K^2)
 */

// This algorithm is used to solve recurrences such as:
//   f(n) = x1 * f(n - 1) + x2 * f(n - 1) + ... + xk * f(n - k)
//
// It works by defining this recurrence as a linear combination,
// for example (k = 2):
//   f(n) = [x1  x2] [f(n - 1)]
//                   [f(n - 2)]
// It can be rewriten as:
//   [  f(n)  ] = [x1  x2] [f(n - 1)]
//   [f(n - 1)]   [ 1   0] [f(n - 2)]
//
// And that is solved by calculating the following matrix power:
//   [x1  x2]^n
//   [ 1   0]

#define K 2

struct mat {
  ll m[K][K];

  // Matrix multiplication - O(k^3)
  mat operator*(mat a) {
    mat aux;

    for(int i = 0; i < K; i++)
      for(int j = 0; j < K; j++) {
        ll sum = 0;
        for(int k = 0; k < K; k++)
          sum += (m[i][k] * a.m[k][j]) % MOD;
        aux.m[i][j] = sum % MOD;
      }

    return aux;
  }
};


// Fast exponentiation (can be used with integers as well) - O(log n)
ll mat_pow(mat in, ll n) {
  mat ans, b = in;

  // Set ans as identity matrix
  memset(ans.m, 0, sizeof ans.m);
  for (int i = 0; i < K; ++i)
    ans.m[i][i] = 1;

  while (n) {
    if (n & 1)
      ans = ans * b;

    n >>= 1;
    b = b * b;
  }

  return ans;
}
